# -*- coding: utf-8 -*-

import pickle
import re
import logging

from inspect import (currentframe, getargvalues, getouterframes, getargspec,
                     ismethod, isfunction, isclass)
from copy import copy
from collections import OrderedDict

import numpy as np

try:
    from collections import OrderedDict
except ImportError:
    from pyhrf.tools.backports import OrderedDict

import pyhrf
from pyhrf.tools import PickleableStaticMethod


logger = logging.getLogger(__name__)


NEWLINE_TAG = '##'


class DeprecatedXMLFormatException(Exception):
    pass


def protect_xml_attr(sa):
    if NEWLINE_TAG in sa:
        raise Exception('Cannot safely protect new line chars')
    return sa.replace('\n', NEWLINE_TAG)


def unprotect_xml_attr(sa):
    return sa.replace(NEWLINE_TAG, '\n')


def numpy_array_from_string(s, sdtype, sshape=None):
    if sdtype[:3] == 'str':
        sdtype = 'str'
    if 'unicode' in sdtype:
        sdtype = 'unicode'

    data = str(s).split()
    if sdtype != 'str' and sdtype != 'unicode':
        data = [float(e) for e in data]

    a = np.array(data, dtype=np.typeDict[sdtype])
    if sshape is not None:
        a.shape = tuple(int(e) for e in sshape.strip('(),').split(','))
    return a


class Initable(object):

    """
    Abstract class to keep track of how an object is initialised.
    To do so, it stores the init function and its parameters.
    The aim is to use it in a user interface or to serialize objects.
    It also allows to add comments and meta info on init parameters.
    """
    _init_obj = None
    _init_parameters = None

    def __init__(self):
        if self._init_obj is None:
            # gathers all names and values for init parameters:
            self._init_parameters = OrderedDict()

            # function used to instanciate the current object:
            self._init_obj = self.__class__

            # inspect the previous frame to get the values of the
            # the init parameters:
            frame, _, _, _, _, _ = getouterframes(currentframe())[1]
            args, _, _, values = getargvalues(frame)
            logger.debug('self.__class__: %s\nargs: %s\nvalues: %s',
                         str(self.__class__), str(args), str(values))

            values.pop('self', None)
            if 'self' in args:
                args.remove('self')
            for k in args:
                self._init_parameters[k] = copy(values[k])

            logger.debug(
                'self._init_parameters: %s', str(self._init_parameters))

            #
            if hasattr(self, 'parametersComments'):
                ip = self._init_parameters.keys()
                pc = self.parametersComments
                pc = dict((k, v) for k, v in pc.items() if k in ip)
                self.parametersComments = pc
            #     for pname in self.parametersComments.keys():
            #         ip = self._init_parameters.keys()
            #         if pname not in ip:
            #             raise Exception('Entry "%s" in parametersComments is '\
            #                             'not in init parameters (%s) of %s' \
            #                             %(pname,','.join(ip),
            #                               self.get_init_func()))

            if hasattr(self, 'parametersToShow'):
                ip = self._init_parameters.keys()
                for pname in self.parametersToShow:
                    if pname not in ip:
                        raise Exception('Entry "%s" in parametersToShow is not'
                                        ' in init parameters (%s)'
                                        % (pname, ','.join(ip)))

            # translations of method arguments for user interface:
            self.arg_translation = {}
            self.arg_itranslation = {}

    def get_init_func(self):
        if isinstance(self._init_obj, PickleableStaticMethod):
            iobj = self._init_obj.fn
        else:
            iobj = self._init_obj
        return iobj

    def check_init_obj(self, params=None):
        """ check if the function used for init can be used in this API
        -> must allow \*\*kwargs and \*args.
        All arguments must have a value: either a default one or specified in
        the input dict *params*
        """
        ifunc = self.get_init_func()
        args, varargs, varkw, defaults = getargspec(ifunc)
        if varkw is not None:  # we don't want **kwargs arguments
            raise Exception('Keywords dict argument (eg **kwargs) '
                            'not supported (init function:%s).'
                            % str(ifunc))
        if varargs is not None:  # we don't want *args arguments
            raise Exception('Positional list argument (eg *args) '
                            ' not supported (init function:%s)'
                            % str(ifunc))
        if args[0] == 'self' or args[0] == 'cls':
            args = args[1:]
        if params is not None:
            if set(params.keys()) != set(args):
                raise Exception('Some arguments do not '
                                'have a value: %s'
                                % (str(set(args).difference(params.keys()))))

        elif len(args) - 1 != len(defaults):
            pos_args = args[:len(args) - len(defaults)]
            print 'pos_args:', pos_args
            raise Exception('In init function "%s.%s", some arguments are '
                            'not keywords arguments: %s'
                            % (str(self.__class__),
                               str(ifunc.__name__),
                               ','.join(pos_args)))

    def set_init_param(self, param_name, param_value):
        if not self._init_parameters.has_key(param_name):
            raise Exception('"%s" is not an argument of init function %s'
                            % (param_name, self.get_init_func()))
        self._init_parameters[param_name] = copy(param_value)

    def set_init(self, init_obj, **init_params):
        """ Override init function with *init_obj* and use *init_params*
        as new init parameters. *init_obj* must return an instance of the
        same class as the current object. Useful when the object is not
        instanciated via its __init__ function but eg a static method.
        """
        self._init_obj = init_obj
        ifunc = self.get_init_func()
        args, varargs, varkw, defaults = getargspec(ifunc)

        self.check_init_obj(init_params)
        self._init_parameters = dict((k, copy(v))
                                     for k, v in init_params.iteritems())

        for ip in self._init_parameters:
            if ip not in args:
                raise Exception('Init parameter "%s" is not an argument of '
                                'init function "%s". Args are: %s'
                                % (ip, str(ifunc), ','.join(args)))

    def get_parameters_comments(self):
        logger.debug('get_parameters_comments...')
        return getattr(self, 'parametersComments', {})

    def get_parameters_meta(self):
        logger.debug('get_parameters_meta...')
        # TODO
        return {}

    def get_parameters_to_show(self):
        logger.debug('get_parameters_to_show...')
        return getattr(self, 'parametersToShow', [])

    def init_new_obj(self):
        """ Creates a new instance
        """
        # if self._init_obj.__name__ == '__init__':
        #     return self._init_obj.im_self.__class__(**self._init_parameters)
        # else:

        return self._init_obj(**self._init_parameters)

    def assert_is_initialized(self):
        if self._init_obj is None or self._init_parameters is None:
            cname = self.__class__.__name__
            raise Exception('%s instance is not properly initialized for the '
                            'Initable API. Did you call Initable.__init__ in '
                            '%s.__init__ ?' % (cname, cname))

    def to_ui_node(self, label, parent=None):
        self.assert_is_initialized()
        logger.debug('Initable.to_ui_node(label=%s) ...', label)
        n = UiNode(
            label, parent, {'init_obj': self._init_obj, 'type': 'Initable'})

        for pname, pval in self._init_parameters.iteritems():
            logger.debug('pname: %s, pval: %s', str(pname), str(pval))
            n.add_child(UiNode.from_py_object(pname, pval))
        return n

    @PickleableStaticMethod
    def from_ui_node(self, node):
        logger.debug('Initable.from_ui_node [%s]', node.label())
        init_obj = node.get_attribute('init_obj')
        node_type = node.get_attribute('type')
        if init_obj is not None:
            params = dict((c.label(), self.from_ui_node(c))
                          for c in node.get_children())
            logger.debug('-> init with params: %s', str(params))
            if hasattr(init_obj, '__name__') and init_obj.__name__ == '__init__':
                return init_obj.im_self.__class__(**params)
            else:
                return init_obj(**params)
        elif node_type is not None:
            logger.debug('-> typped value (%s)', str(node_type))
            if isinstance(node_type, str):
                if node_type == 'ndarray':
                    sdtype = node.get_attribute('dtype')
                    if sdtype.isdigit():
                        sdtype = int(sdtype)
                    sh = node.get_attribute('shape')
                    logger.debug('node: %s', node.log())
                    if node.childCount() > 0:
                        array_data = node.child(0)._label
                    else:  # 0d array
                        array_data = ''
                    return numpy_array_from_string(array_data, sdtype, sh)
                elif node_type == 'list':
                    return list(self.from_ui_node(c)
                                for c in node.get_children())
                elif node_type == 'tuple':
                    return tuple(self.from_ui_node(c)
                                 for c in node.get_children())
                elif node_type in ['odict', 'dict']:
                    dclass = [OrderedDict, dict][node_type == 'dict']
                    return dclass((c.label(), self.from_ui_node(c))
                                  for c in node.get_children())
            elif node_type == bool:
                return node.child(0)._label == 'True'
            elif node_type == str and node.childCount() == 0:  # emtpy string
                return ''
            else:
                logger.debug('casting "%s" to node_type %s ...',
                             node.child(0)._label, str(node_type))
                r = node_type(node.child(0)._label)
                logger.debug('-> %s', str(r))
                return r
        else:
            l = node.label()
            logger.debug('-> direct value: %s', str(l))
            return l

    def set_arg_translation(self, a, t):
        """ Set the display name of argument *a* as *t*
        """
        self.arg_translation[a] = t
        self.arg_itranslation[t] = a

    def get_arg_for_ui(self, a):
        return self.arg_translation.get(a, a)

    def get_arg_from_ui(self, a):
        return self.arg_itranslation.get(a, a)

XmlInitable = Initable  # just an alias


class UiNode(object):
    """
    Store data hierarchically to be used in a Qt model/tree view setting.
    Also store additional node-specific data as attributes (in a dict).
    Attributes must only contain strings.

    The resulting data structure is:

    .. code-block:: none

           col 0        | col 1
        |- <node_label> | <node_attributes>                   #row 0
            |
            |  col 0              | col 1
            |- <child_node_label> | <child_node_attributes>   #row 0
                |
                |...
        ...

    This structure is similar to DOM.

    Features:

    - can be instantiated from any python object that can be serialized into
      human-readable strings: bool, int, float, string, numpy scalars,
      numpy arrays. It also support container types (list, dict, OrderedDict)
      as long as their items are also serializable into human-readable strings.

    See static method *from_py_object*.

    .. todo::

        set and tuple

    .. todo::

        test unicode
    """

    def __init__(self, label, parent=None, attributes=None):

        super(UiNode, self).__init__()

        logger.debug('Create new UiNode(%s,%s,%s)',
                     label, str(parent), str(attributes))

        self._label = label
        self._children = []
        self._parent = parent
        if attributes is None:
            attributes = {}

        # Check input attributes (has to be a dict of strings):
        if not isinstance(attributes, dict):
            raise Exception('Wrong type "%s" for attributes, has to be a dict'
                            % str(type(attributes)))

        for k, v in attributes.iteritems():
            if not isinstance(k, str):
                raise Exception('Wrong type for attribute "%s" '
                                '(has to be string)' % str(k))
            if k not in ['init_obj', 'type'] and not isinstance(v, str):
                # init_obj can be a method/function (see Initable class)
                raise Exception('Wrong type for value of attribute "%s" (%s) '
                                '(has to be string)' % (str(k), str(v)))

        self._attributes = attributes

        if parent is not None:
            parent.add_child(self)

    def set_attribute(self, attr_name, attr_value):
        self._attributes[attr_name] = attr_value

    def get_attribute(self, attr_name):
        return self._attributes.get(attr_name)

    def has_attribute(self, attr_name):
        return self._attributes.has_key(attr_name)

    @PickleableStaticMethod
    def _serialize_init_obj(self, f):
        """ Return a dict of strings describing function f.
        """
        logger.debug('serialize init func %s', str(f))

        if not isclass(f) and hasattr(f, 'im_func') and \
                f.im_func.__name__ == '__init__':
            f = f.im_class

        return {'pickled_init_obj': protect_xml_attr(pickle.dumps(f))}

        # if isinstance(f, PickleableStaticMethod):
        #     f = f.fn
        # return {'init_python_class' : f.__class__.__name__,

    @PickleableStaticMethod
    def _unserialize_init_obj(self, d, pop=False):
        """ Return a function from info contained in dict *d*
        """
        if pop:
            pf = d.pop('pickled_init_obj', None)
        else:
            pf = d.get('pickled_init_obj')
        if pf is not None:
            return pickle.loads(unprotect_xml_attr(pf))
        else:
            return None

    def _serialize_type(self, t):
        """ Return a dict of strings describing type t.
        """
        return {'pickled_type': protect_xml_attr(pickle.dumps(t))}

    @PickleableStaticMethod
    def _unserialize_type(self, d, pop=False):
        if pop:
            pt = d.pop('pickled_type', None)
        else:
            pt = d.get('pickled_type')
        if pt is not None:
            return pickle.loads(unprotect_xml_attr(pt))
        else:
            return None

    def serialize_attributes(self):
        a = {}
        for k, v in self._attributes.iteritems():
            if k == 'init_obj':
                a.update(self._serialize_init_obj(v))
            elif k == 'type':
                a.update(self._serialize_type(v))
            else:
                a[k] = v
        return a

    @PickleableStaticMethod
    def unserialize_attributes(self, a):
        dest = a.copy()  # make a copy 'cause things will be popping
        logger.debug('unserialize_attributes: %s', str(a))
        for k, v in a.items():
            init_obj = self._unserialize_init_obj(dest, pop=True)
            if init_obj is not None:
                dest['init_obj'] = init_obj
                dest['type'] = 'Initable'
            else:
                t = self._unserialize_type(dest, pop=True)
                if t is not None:
                    dest['type'] = t

        return dest

    def set_label(self, label):
        self._label = label
        return True

    @PickleableStaticMethod
    def _pyobj_has_leaf_type(self, o):
        """ Return true if object *o* can be stored in a leaf Node, ie
        if it can be serialized in a simple human-readable string.
        """
        return isinstance(o, (int, float, str, np.ndarray)) or \
            np.isscalar(o) or o is None
        # ((isinstance(o, list) or isinstance(o, list)) and \
        #  all([isinstance(e, (int, float, str)) for e in o])) or \
    # or \
    #        (isinstance(o, (dict, OrderedDict)) and \
    #         all([isinstance(e, (int, float, str)) for e in o.values()]))

    @PickleableStaticMethod
    def from_py_object(self, label, obj, parent=None):
        """
        Convert a python object into a UiNode object.
        Types that are supported for *obj*:
            - NoneType
            - int
            - float
            - str
            - list of supported types
            - dict of supported types
            - OrderedDict
            - numpy.ndarray
            - Initable
        """
        logger.debug('UiNode.from_py_object(label=%s,obj=%s) ...',
                     label, str(obj))

        if isinstance(obj, Initable):
            n = obj.to_ui_node(label, parent)
        else:
            if UiNode._pyobj_has_leaf_type(obj):
                if isinstance(obj, np.ndarray):
                    dt = str(obj.dtype.name)
                    sh = str(obj.shape)
                    n = UiNode(label, attributes={'type': 'ndarray',
                                                  'dtype': dt,
                                                  'shape': sh})
                    # string representation of the flat array:
                    s = ' '.join(str(e) for e in obj.ravel())
                    n.add_child(UiNode(s))
                elif obj is None:
                    n = UiNode(label, attributes={'type': 'None'})
                    n.add_child(UiNode('None'))
                else:
                    n = UiNode(label, attributes={'type': obj.__class__})
                    n.add_child(UiNode(str(obj)))
            elif isinstance(obj, list):
                n = UiNode(label, attributes={'type': 'list'})
                for i, sub_val in enumerate(obj):
                    n.add_child(UiNode.from_py_object('item%d' % i, sub_val))
            elif isinstance(obj, tuple):
                n = UiNode(label, attributes={'type': 'tuple'})
                for i, sub_val in enumerate(obj):
                    n.add_child(UiNode.from_py_object('item%d' % i, sub_val))
            elif isinstance(obj, (dict, OrderedDict)):
                t = ['odict', 'dict'][obj.__class__ == dict]
                n = UiNode(label, attributes={'type': t})
                for k, v in obj.iteritems():
                    n.add_child(UiNode.from_py_object(k, v))
            else:
                raise Exception('In UiNode.from_py_object, unsupported object: '
                                '%s (type: %s)' % (str(obj), str(type(obj))))
        return n

    def label(self):
        return self._label

    def type_info(self):
        return self._attributes.get('type')

    def to_xml(self, pretty=False):
        """
        Return an XML representation (str) of the Node and its children.
        """
        from xml.dom.minidom import Document
        doc = Document()
        node = doc.createElement(self.label())
        doc.appendChild(node)
        for k, v in self.serialize_attributes().iteritems():
            node.setAttribute(k, v)

        for c in self.get_children():
            c._recurse_to_xml(doc, node)

        if pretty:
            # TODO: manage encoding stuff?
            return doc.toprettyxml(indent='    ')
        else:
            return doc.toxml()

    def is_leaf_node(self):
        return len(self.get_children()) == 0 and len(self._attributes) == 0

    def _recurse_to_xml(self, doc, parent):
        logger.debug('_recurse_to_xml ...')

        if not self.is_leaf_node():
            node = doc.createElement(self.label())
        else:  # leaf node -> use TextNode
            logger.debug('text node: %s', self.label())
            node = doc.createTextNode(self.label())

        parent.appendChild(node)

        for k, v in self.serialize_attributes().iteritems():
            node.setAttribute(k, v)

        for c in self.get_children():
            c._recurse_to_xml(doc, node)

    @PickleableStaticMethod
    def from_xml(self, sxml):
        from xml.dom.minidom import parseString
        doc = parseString(str(sxml))  # TODO: ingore tab, \n
        return self._recurse_from_xml(doc.documentElement)

    @PickleableStaticMethod
    def _recurse_from_xml(self, node):
        logger.debug('_recurse_from_xml node -> tag=%s, val=%s, type=%s ',
                     str(getattr(node, 'tagName', 'N/A')), str(node.nodeValue),
                     str(node.nodeType))
        if node.nodeType == node.TEXT_NODE and \
                re.match('^[ \n]*$', node.wholeText):  # empty node
            return None
        a = node.attributes
        if a is not None:
            attributes = OrderedDict((str(a.item(i).nodeName),
                                      str(a.item(i).nodeValue))
                                     for i in range(a.length))
            attributes = self.unserialize_attributes(attributes)
        else:
            attributes = None

        # Manage deprecated API:
        if getattr(node, 'tagName', 'N/A') == 'root' and \
                attributes is not None and attributes.has_key('pythonXMLHandlerModule'):
            raise DeprecatedXMLFormatException(
                'Deprecated XML format (root node)')

        node_name = str(node.nodeName)
        if node_name != '#text':
            n = UiNode(node_name, attributes=attributes)
        else:
            n = UiNode(node.nodeValue, attributes=attributes)

        child_nodes = node.childNodes
        logger.debug('create new node %s (-> %d children)',
                     node_name, len(child_nodes))
        logger.debug('node value: %s', str(node.nodeValue))
        for c in child_nodes:
            rc = self._recurse_from_xml(c)
            if rc is not None:
                n.add_child(rc)

        return n

    def log(self, tabLevel=-1):

        output = ""
        tabLevel += 1

        for i in range(tabLevel):
            output += "\t"

        output += "|------" + self._label
        if len(self._attributes) > 0:
            output += ' |'
            for k, v in self._attributes.iteritems():
                if isfunction(v) or ismethod(v):
                    v = v.__name__
                elif hasattr(v, 'im_func'):
                    v = v.im_func
                output += ' %s:%s' % (str(k), str(v))
        output += '\n'

        for child in self._children:
            output += child.log(tabLevel)

        tabLevel -= 1
        output += "\n"

        return output

    def get_children(self):
        return self._children

    def add_child(self, child):
        self._children.append(child)
        child._parent = self

    def childCount(self):
        return len(self._children)

    def child(self, row):
        return self._children[row]


def read_xml(fn):
    f = open(fn)
    xml = f.read()
    f.close()
    return from_xml(xml)


def write_xml(obj, fn):
    f = open(fn, 'w')
    f.write(to_xml(obj))
    f.close()


def to_xml(obj, label='anonym', pretty=False):
    """
    Return an XML representation of the init state of the given object *obj*.
    """
    return UiNode.from_py_object(label, obj).to_xml(pretty)


def from_xml(sxml):
    return Initable.from_ui_node(UiNode.from_xml(sxml))



# -*- coding: utf-8 -*-

import logging

import numpy as np

from numpy.random import randn, rand, permutation

import pyhrf
from pyhrf.jde.samplerbase import *
from pyhrf import xmlio
from pyhrf.jde.intensivecalc import (sampleSmmNrl2, sampleSmmNrlBar,
                                     sampleSmmNrl2WithRelVar_NEW)
from pyhrf.jde.nrl.bigaussian import NRLSampler, NRLSamplerWithRelVar


logger = logging.getLogger(__name__)


class NRL_Drift_Sampler(NRLSampler):
    """
    Class handling the Gibbs sampling of Neural Response Levels in the case of
    joint drift sampling.
    """

    def computeVarYTildeOpt(self, varXh):
        NRLSampler.computeVarYTildeOpt(self, varXh)
        matPl = self.samplerEngine.get_variable('drift').matPl
        self.varYbar = self.varYtilde - matPl

    def sampleNextInternal(self, variables):
        # TODO : comment
        sIMixtP = variables[self.samplerEngine.I_MIXT_PARAM]
        varCI = sIMixtP.currentValue[sIMixtP.I_VAR_CI]
        varCA = sIMixtP.currentValue[sIMixtP.I_VAR_CA]
        meanCA = sIMixtP.currentValue[sIMixtP.I_MEAN_CA]
        rb = variables[self.samplerEngine.I_NOISE_VAR].currentValue
        sHrf = self.get_variable('hrf')
        h = sHrf.currentValue
        varXh = sHrf.varXh

        logger.debug('varXh %s :', str(varXh.shape))
        logger.debug(varXh)

        self.computeVarYTildeOpt(varXh)

        self.labelsSamples = rand(self.nbConditions, self.nbVox)
        self.nrlsSamples = randn(self.nbConditions, self.nbVox)

        gTg = np.diag(np.dot(varXh.transpose(), varXh))

        if self.imm:
            raise NotImplementedError(
                "IMM with drift sampling is not available")
        else:  # smm
            self.sampleNrlsSerial(rb, h, varCI, varCA, meanCA, gTg, variables)
            self.computeVarYTildeOpt(varXh)

        if ((self.currentValue >= 1000).any() and
                pyhrf.__usemode__ == pyhrf.DEVEL):
            logger.info("Weird NRL values detected ! %d/%d",
                        (self.currentValue >= 1000).sum(),
                        self.nbVox * self.nbConditions)

        if logger.getEffectiveLevel() <= logging.INFO:
            self.reportDetection()

        self.computeAA(self.currentValue, self.aa)

        self.printState(4)
        self.iteration += 1  # TODO : factorize !!

    def sampleNrlsSerial(self, rb, h, varCI, varCA, meanCA,
                         gTg, variables):

        logger.info('Sampling Nrls (serial, spatial prior) ...')
        sIMixtP = variables[self.samplerEngine.I_MIXT_PARAM]
        var = sIMixtP.getCurrentVars()
        mean = sIMixtP.getCurrentMeans()
        rb = variables[self.samplerEngine.I_NOISE_VAR].currentValue

        # Add one dimension to be consistent with habituation model
        varXh = self.get_variable('hrf').varXh
        varXht = varXh.transpose()
        nrls = self.currentValue

        neighbours = self.dataInput.neighboursIndexes

        beta = self.samplerEngine.get_variable('beta').currentValue
        voxOrder = permutation(self.nbVox)

        sampleSmmNrl2(voxOrder.astype(np.int32), rb.astype(np.float64),
                      neighbours.astype(np.int32), self.varYbar,
                      self.labels, np.array([varXh], dtype=np.float64),
                      self.currentValue,
                      self.nrlsSamples.astype(np.float64),
                      self.labelsSamples.astype(np.float64),
                      np.array([varXht], dtype=np.float64),
                      gTg.astype(np.float64),
                      beta.astype(np.float64), mean.astype(np.float64),
                      var.astype(np.float64), self.meanClassApost,
                      self.varClassApost, self.nbClasses,
                      self.sampleLabelsFlag + 0, self.iteration,
                      self.nbConditions)

        self.countLabels(self.labels, self.voxIdx, self.cardClass)


class NRL_Drift_SamplerWithRelVar(NRLSamplerWithRelVar):
    """
    Class handling the Gibbs sampling of Neural Response Levels in the case of
    joint drift sampling and relevant variable.
    """

    def computeVarYTildeOptWithRelVar(self, varXh, w):
        NRLSamplerWithRelVar.computeVarYTildeOptWithRelVar(self, varXh, w)
        matPl = self.samplerEngine.get_variable('drift').matPl
        self.varYbar = self.varYtilde - matPl

    def sampleNextInternal(self, variables):

        # TODO : comment
        sIMixtP = variables[self.samplerEngine.I_MIXT_PARAM]
        varCI = sIMixtP.currentValue[sIMixtP.I_VAR_CI]
        varCA = sIMixtP.currentValue[sIMixtP.I_VAR_CA]
        meanCA = sIMixtP.currentValue[sIMixtP.I_MEAN_CA]
        rb = variables[self.samplerEngine.I_NOISE_VAR].currentValue
        sHrf = self.get_variable('hrf')
        h = sHrf.currentValue
        sDrift = self.get_variable('drift')
        varXh = sHrf.varXh
        varLambda = variables[
            self.samplerEngine.I_WEIGHTING_PROBA].currentValue
        w = self.get_variable('W').currentValue

        t1 = self.get_variable('W').t1
        t2 = self.get_variable('W').t2

        logger.debug('varXh %s :', str(varXh.shape))
        logger.debug(varXh)

        self.computeVarYTildeOptWithRelVar(varXh, w)

        self.labelsSamples = rand(self.nbConditions, self.nbVox)
        self.nrlsSamples = randn(self.nbConditions, self.nbVox)

        gTg = np.diag(np.dot(varXh.transpose(), varXh))

        if self.imm:
            raise NotImplementedError(
                "IMM with drift sampling is not available")
        else:  # smm
            self.sampleNrlsSerialWithRelVar(rb, h, gTg, variables, w, t1, t2)
            self.computeVarYTildeOptWithRelVar(varXh, w)

        if ((self.currentValue >= 1000).any() and
                pyhrf.__usemode__ == pyhrf.DEVEL):
            logger.info("Weird NRL values detected ! %d/%d",
                        (self.currentValue >= 1000).sum(),
                        self.nbVox * self.nbConditions)

        if logger.getEffectiveLevel() <= logging.INFO:
            self.reportDetection()

        self.computeAA(self.currentValue, self.aa)

        wa = np.zeros((self.nbConditions, self.nbVox))
        self.computeWA(self.currentValue, w, wa)
        self.computeSumWAxh(wa, varXh)

        self.printState(4)
        self.iteration += 1  # TODO : factorize !!

    def sampleNrlsSerialWithRelVar(self, rb, h, gTg, variables, w, t1, t2):

        logger.info('Sampling Nrls (serial, spatial prior) ...')
        sIMixtP = variables[self.samplerEngine.I_MIXT_PARAM]
        var = sIMixtP.getCurrentVars()
        mean = sIMixtP.getCurrentMeans()
        rb = variables[self.samplerEngine.I_NOISE_VAR].currentValue
        y = self.dataInput.varMBY
        matPl = self.samplerEngine.get_variable('drift').matPl
        sampleWFlag = self.get_variable('W').sampleFlag

        # Add one dimension to be consistent with habituation model
        varXh = self.get_variable('hrf').varXh
        varXht = varXh.transpose()
        nrls = self.currentValue

        neighbours = self.dataInput.neighboursIndexes

        beta = self.samplerEngine.get_variable('beta').currentValue
        voxOrder = permutation(self.nbVox)

        cardClassCA = np.zeros(self.nbConditions, dtype=int)
        for i in range(self.nbConditions):
            cardClassCA[i] = self.cardClass[self.L_CA, i]

        sampleSmmNrl2WithRelVar_NEW(voxOrder.astype(np.int32),
                                    rb.astype(np.float64),
                                    neighbours.astype(np.int32), self.varYbar,
                                    y.astype(np.float64),
                                    matPl.astype(np.float64), self.labels,
                                    np.array([varXh], dtype=np.float64),
                                    self.currentValue,
                                    self.nrlsSamples.astype(np.float64),
                                    self.labelsSamples.astype(np.float64),
                                    np.array([varXht], dtype=np.float64),
                                    gTg.astype(np.float64),
                                    beta.astype(np.float64),
                                    mean.astype(np.float64),
                                    var.astype(np.float64),
                                    self.meanClassApost, self.varClassApost,
                                    w.astype(np.int32), t1, t2,
                                    cardClassCA.astype(
                                        np.int32), self.nbClasses,
                                    self.sampleLabelsFlag + 0, self.iteration,
                                    self.nbConditions, sampleWFlag)

        self.countLabels(self.labels, self.voxIdx, self.cardClass)


class NRLsBar_Drift_Multi_Sess_Sampler(NRLSampler):
    """
    Class handling the Gibbs sampling of Neural Response Levels in the case of
    joint drift sampling.
    """

    def linkToData(self, dataInput):
        self.dataInput = dataInput
        self.nbConditions = self.dataInput.nbConditions
        self.nbVox = self.dataInput.nbVoxels
        self.ny = self.dataInput.ny
        self.nbSessions = self.dataInput.nbSessions
        self.cardClass = np.zeros(
            (self.nbClasses, self.nbConditions), dtype=int)
        self.voxIdx = [range(self.nbConditions)
                       for c in xrange(self.nbClasses)]

        # TODO handle condition matching
        if dataInput.simulData is not None:
            if isinstance(dataInput.simulData, dict):
                if dataInput.simulData.has_key('nrls'):
                    nrls = dataInput.simulData['nrls']
                    if isinstance(nrls, xndarray):
                        self.trueValue = nrls.reorient(
                            ['condition', 'voxel']).data
                    else:
                        self.trueValue = nrls
                if dataInput.simulData.has_key('labels'):
                    labels = dataInput.simulData['labels']
                    if isinstance(labels, xndarray):
                        self.trueLabels = labels.reorient(
                            ['condition', 'voxel']).data
                    else:
                        self.trueLabels = labels
            else:
                sd = dataInput.simulData[0]
                self.trueValue = sd['nrls'].astype(np.float64)
                self.trueLabels = sd['labels'].astype(np.int32)

            self.trueLabels = self.trueLabels[
                :self.nbConditions, :].astype(np.int32)
            self.trueValue = self.trueValue[
                :self.nbConditions, :].astype(np.float64)
        else:
            self.trueLabels = None

    def checkAndSetInitValue(self, variables):
        NRLSampler.checkAndSetInitLabels(self, variables)
        NRLSampler.checkAndSetInitNRL(self, variables)
        mixt_par = self.get_variable('mixt_params')
        mixt_par.checkAndSetInitValue(variables)
        weights_par = variables[self.samplerEngine.I_WEIGHTING_PROBA_NRLS_BAR]
        weights_par.checkAndSetInitValue(variables)

    def sampleNextAlt(self, variables):
        pass

    def samplingWarmUp(self, variables):
        """
        #TODO : comment
        """

        # Precalculations and allocations :

        self.imm = self.samplerEngine.get_variable('beta').currentValue[0] < 0

        self.varClassApost = np.zeros((self.nbClasses, self.nbConditions, self.nbVox),
                                      dtype=np.float64)
        self.sigClassApost = np.zeros((self.nbClasses, self.nbConditions, self.nbVox),
                                      dtype=float)
        self.meanClassApost = np.zeros((self.nbClasses, self.nbConditions,
                                        self.nbVox), dtype=np.float64)
        self.meanApost = np.zeros((self.nbConditions, self.nbVox), dtype=float)
        self.sigApost = np.zeros((self.nbConditions, self.nbVox), dtype=float)

        self.iteration = 0

        self.countLabels(self.labels, self.voxIdx, self.cardClass)

    def sampleNextInternal(self, variables):
        # TODO : comment
        sIMixtP = self.get_variable('mixt_params')
        varCI = sIMixtP.currentValue[sIMixtP.I_VAR_CI]
        varCA = sIMixtP.currentValue[sIMixtP.I_VAR_CA]
        meanCA = sIMixtP.currentValue[sIMixtP.I_MEAN_CA]

        self.labelsSamples = rand(self.nbConditions, self.nbVox)
        self.nrlsSamples = randn(self.nbConditions, self.nbVox)

        if self.imm:
            raise NotImplementedError(
                "IMM with drift sampling is not available")
        else:  # smm
            self.sampleNrlsSerial(varCI, varCA, meanCA, variables)

        if ((self.currentValue >= 1000).any() and
                pyhrf.__usemode__ == pyhrf.DEVEL):
            logger.info("Weird NRL values detected ! %d/%d",
                        (self.currentValue >= 1000).sum(),
                        self.nbVox * self.nbConditions)

        if logger.getEffectiveLevel() <= logging.INFO:
            self.reportDetection()

        self.printState(4)
        self.iteration += 1  # TODO : factorize !!

    def sampleNrlsSerial(self, varCI, varCA, meanCA,
                         variables):

        logger.info('Sampling Nrls (serial, spatial prior) ...')
        sIMixtP = variables[self.samplerEngine.I_MIXT_PARAM_NRLS_BAR]
        nrl_var_sess = self.samplerEngine.get_variable(
            'variance_nrls_by_session').currentValue
        sum_nrl_sess = self.samplerEngine.get_variable(
            'nrl_by_session').currentValue.sum(0)

        var = sIMixtP.getCurrentVars()
        mean = sIMixtP.getCurrentMeans()

        neighbours = self.dataInput.neighboursIndexes

        beta = self.samplerEngine.get_variable('beta').currentValue
        voxOrder = permutation(self.nbVox)

        logger.info('Labels sampling: %s', str(self.sampleLabelsFlag))

        sampleSmmNrlBar(voxOrder.astype(np.int32),
                        neighbours.astype(np.int32),
                        self.labels, self.currentValue,
                        self.nrlsSamples.astype(np.float64),
                        self.labelsSamples.astype(np.float64),
                        beta.astype(np.float64), mean.astype(np.float64),
                        var.astype(np.float64), self.meanClassApost,
                        self.varClassApost, nrl_var_sess, sum_nrl_sess,
                        self.nbClasses,
                        self.sampleLabelsFlag + 0, self.iteration,
                        self.nbConditions, self.nbSessions)

        self.countLabels(self.labels, self.voxIdx, self.cardClass)

# Multi-sessions case


class BiGaussMixtureParams_Multi_Sess_NRLsBar_Sampler(GibbsSamplerVariable):
    """
    #TODO : comment

    """

    I_MEAN_CA = 0
    I_VAR_CA = 1
    I_VAR_CI = 2
    NB_PARAMS = 3
    PARAMS_NAMES = ['Mean_Activ', 'Var_Activ', 'Var_Inactiv']

    P_VAL_INI = 'initialValue'
    P_SAMPLE_FLAG = 'sampleFlag'
    P_USE_TRUE_VALUE = 'useTrueValue'

    P_MEAN_CA_PR_MEAN = 'meanCAPrMean'
    P_MEAN_CA_PR_VAR = 'meanCAPrVar'

    P_VAR_CI_PR_ALPHA = 'varCIPrAlpha'
    P_VAR_CI_PR_BETA = 'varCIPrBeta'

    P_VAR_CA_PR_ALPHA = 'varCAPrAlpha'
    P_VAR_CA_PR_BETA = 'varCAPrBeta'

    P_HYPER_PRIOR = 'hyperPriorType'

    P_ACTIV_THRESH = 'mean_activation_threshold'

    #"peaked" priors
    defaultParameters = {
        P_VAL_INI: None,
        P_SAMPLE_FLAG: True,
        P_USE_TRUE_VALUE: False,
        P_HYPER_PRIOR: 'Jeffrey',
        P_MEAN_CA_PR_MEAN: 5.,
        P_MEAN_CA_PR_VAR: 20.0,
        P_VAR_CI_PR_ALPHA: 2.04,
        P_VAR_CI_PR_BETA: 2.08,
        P_VAR_CA_PR_ALPHA: 2.01,
        P_VAR_CA_PR_BETA: .5,
        P_ACTIV_THRESH: 4.,
    }

    if pyhrf.__usemode__ == pyhrf.ENDUSER:
        parametersToShow = []

    L_CA = NRLSampler.L_CA
    L_CI = NRLSampler.L_CI

    parametersToShow = [P_VAL_INI, P_SAMPLE_FLAG, P_ACTIV_THRESH,
                        P_USE_TRUE_VALUE,
                        P_HYPER_PRIOR,
                        P_MEAN_CA_PR_MEAN, P_MEAN_CA_PR_VAR, P_VAR_CI_PR_ALPHA,
                        P_VAR_CI_PR_BETA, P_VAR_CA_PR_ALPHA, P_VAR_CA_PR_BETA]

    parametersComments = {
        P_HYPER_PRIOR: "Either 'proper' or 'Jeffrey'",
        P_ACTIV_THRESH: "Threshold for the max activ mean above which the "
        "region is considered activating",
    }

    def __init__(self, parameters=None, xmlHandler=None,
                 xmlLabel=None, xmlComment=None):
        """
        #TODO : comment
        """
        xmlio.XMLParamDrivenClass.__init__(self, parameters, xmlHandler,
                                           xmlLabel, xmlComment)
        sampleFlag = self.parameters[self.P_SAMPLE_FLAG]
        valIni = self.parameters[self.P_VAL_INI]
        useTrueVal = self.parameters[self.P_USE_TRUE_VALUE]

        # get values for priors :
        self.varCIPrAlpha = self.parameters[self.P_VAR_CI_PR_ALPHA]
        self.varCIPrBeta = self.parameters[self.P_VAR_CI_PR_BETA]
        self.varCAPrAlpha = self.parameters[self.P_VAR_CA_PR_ALPHA]
        self.varCAPrBeta = self.parameters[self.P_VAR_CA_PR_BETA]

        self.meanCAPrMean = self.parameters[self.P_MEAN_CA_PR_MEAN]
        self.meanCAPrVar = self.parameters[self.P_MEAN_CA_PR_VAR]

        an = ['component', 'condition']
        ad = {'component': self.PARAMS_NAMES}
        GibbsSamplerVariable.__init__(self, 'mixt_params', valIni=valIni,
                                      useTrueValue=useTrueVal,
                                      sampleFlag=sampleFlag, axes_names=an,
                                      axes_domains=ad)

        php = self.parameters[self.P_HYPER_PRIOR]
        self.hyperPriorFlag = False if php == 'Jeffrey' else True

        self.activ_thresh = self.parameters[self.P_ACTIV_THRESH]

    def linkToData(self, dataInput):
        self.dataInput = dataInput
        self.nbConditions = self.dataInput.nbConditions
        self.nbVox = self.dataInput.nbVoxels
        self.ny = self.dataInput.ny
        self.nbColX = self.dataInput.nbColX

        self.nrlCI = range(self.nbConditions)
        self.nrlCA = range(self.nbConditions)

        print '%%%%%%%%%%%%%%%%%%%%%%%%%%%self.dataInput.simulData : ', self.dataInput.simulData.__class__

        if self.dataInput.simulData is not None and \
                isinstance(self.dataInput.simulData, list):
            print '%%%%%%%%%%:', self.dataInput.simulData[0].has_key('condition_defs')
            if self.dataInput.simulData[0].has_key('condition_defs'):

                # take only 1st session -> same is assumed for others
                cdefs = self.dataInput.simulData[0]['condition_defs']
                self.trueValue = np.zeros((self.NB_PARAMS, self.nbConditions),
                                          dtype=float)
                self.trueValue[self.I_MEAN_CA] = np.array(
                    [c.m_act for c in cdefs])
                self.trueValue[self.I_VAR_CA] = np.array(
                    [c.v_act for c in cdefs])
                self.trueValue[self.I_VAR_CI] = np.array(
                    [c.v_inact for c in cdefs])

        if self.dataInput.simulData is not None:
            if (isinstance(self.dataInput.simulData, BOLDModel) or
                    isinstance(self.dataInput.simulData, BOLDModel2)):
                mixtures = self.dataInput.simulData.nrls.getMixture()
                itemsCond = mixtures.items()
                nbCondInSimu = len(itemsCond)
                self.trueValue = np.zeros((self.NB_PARAMS, nbCondInSimu),
                                          dtype=float)

                meanCA = np.zeros(nbCondInSimu, dtype=float)
                varCA = np.zeros(nbCondInSimu, dtype=float)
                varCI = np.zeros(nbCondInSimu, dtype=float)
                for cn, mixt in mixtures.iteritems():
                    genActiv = mixt.generators['activ']
                    genInactiv = mixt.generators['inactiv']
                    indCond = self.dataInput.simulData.nrls.condIds[cn]
                    varCI[indCond] = genInactiv.std ** 2
                    logger.info('genActiv type = %s', genActiv.type)
                    logger.info('genInactiv type = %s', genInactiv.type)
                    if genActiv.type == 'gaussian' or genActiv.type == 'log-normal':
                        meanCA[indCond] = genActiv.mean
                        varCA[indCond] = genActiv.std ** 2
                    elif genActiv.type == 'gamma':
                        meanCA[indCond] = genActiv.a * genActiv.b
                        varCA[indCond] = genActiv.a * genActiv.b ** 2
                    if 0:
                        # TODO: debug -> why rewritting on meanCA and varCA ?
                        if genInactiv.type == 'gaussian' or genInactiv.type == 'log-normal':
                            meanCA[indCond] = genInactiv.mean
                            varCA[indCond] = genInactiv.std ** 2
                        elif genInactiv.type == 'uniform':
                            meanCA[indCond] = (
                                genInactiv.minV + genInactiv.maxV) / 2.
                            varCA[indCond] = (
                                genInactiv.maxV - genInactiv.minV) / 12.
                        elif genInactiv.type == 'beta':
                            meanCA[indCond] = genInactiv.a /                   \
                                (genInactiv.a + genInactiv.b)
                            varCA[indCond] = genInactiv.a *                   \
                                genInactiv.b / ((genInactiv.a +
                                                 genInactiv.b) ** 2 *
                                                (genInactiv.a +
                                                 genInactiv.b + 1.))
                self.trueValue[self.I_MEAN_CA] = meanCA
                self.trueValue[self.I_VAR_CA] = varCA
                self.trueValue[self.I_VAR_CI] = varCI

            # To handle multi-sessions case
            # self.dataInput.simulData[0] or [1] has same nrls bar values
            if hasattr(self.dataInput, 'nbSessions') and \
                ((isinstance(self.dataInput.simulData[0], BOLDModel) or
                  isinstance(self.dataInput.simulData[0], BOLDModel2))):
                mixtures = self.dataInput.simulData[0].nrls.getMixture()
                itemsCond = mixtures.items()
                nbCondInSimu = len(itemsCond)
                self.trueValue = np.zeros((self.NB_PARAMS, nbCondInSimu),
                                          dtype=float)

                meanCA = np.zeros(nbCondInSimu, dtype=float)
                varCA = np.zeros(nbCondInSimu, dtype=float)
                varCI = np.zeros(nbCondInSimu, dtype=float)
                for cn, mixt in mixtures.iteritems():
                    genActiv = mixt.generators['activ']
                    genInactiv = mixt.generators['inactiv']
                    indCond = self.dataInput.simulData[0].nrls.condIds[cn]
                    varCI[indCond] = genInactiv.std ** 2
                    logger.info('genActiv type = %s', genActiv.type)
                    logger.info('genInactiv type = %s', genInactiv.type)
                    if genActiv.type == 'gaussian' or genActiv.type == 'log-normal':
                        meanCA[indCond] = genActiv.mean
                        varCA[indCond] = genActiv.std ** 2
                    elif genActiv.type == 'gamma':
                        meanCA[indCond] = genActiv.a * genActiv.b
                        varCA[indCond] = genActiv.a * genActiv.b ** 2
                    if 0:
                        # TODO: debug -> why rewritting on meanCA and varCA ?
                        if genInactiv.type == 'gaussian' or genInactiv.type == 'log-normal':
                            meanCA[indCond] = genInactiv.mean
                            varCA[indCond] = genInactiv.std ** 2
                        elif genInactiv.type == 'uniform':
                            meanCA[indCond] = (
                                genInactiv.minV + genInactiv.maxV) / 2.
                            varCA[indCond] = (
                                genInactiv.maxV - genInactiv.minV) / 12.
                        elif genInactiv.type == 'beta':
                            meanCA[indCond] = genInactiv.a /                   \
                                (genInactiv.a + genInactiv.b)
                            varCA[indCond] = genInactiv.a *                   \
                                genInactiv.b / ((genInactiv.a +
                                                 genInactiv.b) ** 2 *
                                                (genInactiv.a +
                                                 genInactiv.b + 1.))
                self.trueValue[self.I_MEAN_CA] = meanCA
                self.trueValue[self.I_VAR_CA] = varCA
                self.trueValue[self.I_VAR_CI] = varCI

        elif self.dataInput.simulData is not None and \
                isinstance(self.dataInput.simulData, dict):

            self.trueValue = np.zeros((self.NB_PARAMS, self.nbConditions),
                                      dtype=float)
            simulation = self.dataInput.simulData
            if simulation.has_key('condition_defs'):
                cdefs = simulation['condition_defs']
                self.trueValue[self.I_MEAN_CA] = np.array(
                    [c.m_act for c in cdefs])
                self.trueValue[self.I_VAR_CA] = np.array(
                    [c.v_act for c in cdefs])
                self.trueValue[self.I_VAR_CI] = np.array(
                    [c.v_inact for c in cdefs])

    def checkAndSetInitValue(self, variables):
        if self.currentValue is None:
            if self.useTrueValue:
                if self.trueValue is not None:
                    # TODO fix condition matching
                    self.currentValue = self.trueValue.copy()[
                        :, :self.nbConditions]
                else:
                    raise Exception('Needed a true value but none defined')

            elif 0 and self.useTrueValue:
                self.trueValue = np.zeros(
                    (self.NB_PARAMS, self.nbConditions), dtype=float)
                self.currentValue = np.zeros(
                    (self.NB_PARAMS, self.nbConditions), dtype=float)
                self.trueValue[self.I_MEAN_CA] = self.ActMeanTrueValue.values()
                self.trueValue[self.I_VAR_CA] = self.ActVarTrueValue.values()
                self.trueValue[self.I_VAR_CI] = self.InactVarTrueValue.values()
                self.currentValue = self.trueValue.copy()[
                    :, :self.nbConditions]

            else:
                nc = self.nbConditions
                self.currentValue = np.zeros(
                    (self.NB_PARAMS, self.nbConditions), dtype=float)
                self.currentValue[self.I_MEAN_CA] = np.zeros(nc) + 30.
                self.currentValue[self.I_VAR_CA] = np.zeros(nc) + 1.
                self.currentValue[self.I_VAR_CI] = np.zeros(nc) + 1.

    def getCurrentVars(self):
        return np.array([self.currentValue[self.I_VAR_CI],
                         self.currentValue[self.I_VAR_CA]])

    def getCurrentMeans(self):
        return np.array([np.zeros(self.nbConditions),
                         self.currentValue[self.I_MEAN_CA]])

    def computeWithProperPriors(self, j, cardCIj, cardCAj):
        if cardCIj > 1:
            nu0j = .5 * np.dot(self.nrlCI[j], self.nrlCI[j])
            varCIj = 1.0 / np.random.gamma(.5 * cardCIj + self.varCIPrAlpha,
                                           1 / (nu0j + self.varCIPrBeta))
        else:
            logger.debug('using only hyper priors for CI (empty class) ...')
            varCIj = 1.0 / \
                np.random.gamma(self.varCIPrAlpha, 1 / self.varCIPrBeta)

        if cardCAj > 1:
            eta1j = np.mean(self.nrlCA[j])
            nrlCACentered = self.nrlCA[
                j] - self.currentValue[self.I_MEAN_CA, j]  # eta1j
            nu1j = .5 * np.dot(nrlCACentered, nrlCACentered)
            varCAj = 1.0 / np.random.gamma(0.5 * cardCAj + self.varCAPrAlpha,
                                           1 / (nu1j + self.varCAPrBeta))
        else:
            logger.debug('using only hyper priors for CA (empty class) ...')
            eta1j = 0.0
            varCAj = 1.0 / \
                np.random.gamma(self.varCAPrAlpha, 1 / self.varCAPrBeta)

        invVarLikelihood = (cardCAj + 0.) / varCAj

        meanCAVarAPost = 1 / (invVarLikelihood + 1 / self.meanCAPrVar)

        rPrMV = self.meanCAPrMean / self.meanCAPrVar
        meanCAMeanAPost = meanCAVarAPost * (eta1j * invVarLikelihood + rPrMV)
        meanCAj = np.random.normal(meanCAMeanAPost, meanCAVarAPost ** 0.5)

        return varCIj, meanCAj, varCAj

    def computeWithJeffreyPriors(self, j, cardCIj, cardCAj):

        logger.info('cond %d - card CI = %d', j, cardCIj)
        logger.info('cond %d - card CA = %d', j, cardCAj)
        logger.info(
            'cond %d - cur mean CA = %f', j, self.currentValue[self.I_MEAN_CA, j])
        if cardCAj > 0:
            logger.info('cond %d - nrl CA: %f(v%f)[%f,%f]', j,
                        self.nrlCA[j].mean(), self.nrlCA[j].var(),
                        self.nrlCA[j].min(), self.nrlCA[j].max())
        if cardCIj > 0:
            logger.info('cond %d - nrl CI: %f(v%f)[%f,%f]', j,
                        self.nrlCI[j].mean(), self.nrlCI[j].var(),
                        self.nrlCI[j].min(), self.nrlCI[j].max())

        if cardCIj > 1:
            nu0j = np.dot(self.nrlCI[j], self.nrlCI[j])
            varCIj = 1.0 / np.random.gamma(0.5 * (cardCIj + 1) - 1, 2. / nu0j)
        else:
            varCIj = 1.0 / np.random.gamma(0.5, 0.2)

        # HACK
        #varCIj = .5

        if cardCAj > 1:
            nrlC1Centered = self.nrlCA[j] - \
                self.currentValue[self.I_MEAN_CA, j]
            nu1j = np.dot(nrlC1Centered, nrlC1Centered)
            logger.info('varCA ~ InvGamma(%f, nu1j/2=%f)', 0.5 * (cardCAj + 1) - 1,
                        nu1j / 2.)
            logger.info(
                ' -> mean = %f', (nu1j / 2.) / (0.5 * (cardCAj + 1) - 1))
            varCAj = 1.0 / np.random.gamma(0.5 * (cardCAj + 1) - 1, 2. / nu1j)
            logger.info('varCAj (j=%d) : %f', j, varCAj)
            if varCAj <= 0.:
                print 'variance for class activ and condition %s '\
                    'is negative or null: %f' % (
                        self.dataInput.cNames[j], varCAj)
                print 'nu1j:', nu1j, '2. / nu1j', 2. / nu1j
                print 'cardCAj:', cardCAj, '0.5 * (cardCAj + 1) - 1:', \
                    0.5 * (cardCAj + 1) - 1
                print '-> setting it to almost 0.'
                varCAj = 0.0001
            eta1j = np.mean(self.nrlCA[j])
            meanCAj = np.random.normal(eta1j, (varCAj / cardCAj) ** 0.5)

            # variance for class activ and condition video is negative or null:
            # 0.000000
            # nu1j: 2.92816412349e-306 2. / nu1j 6.83021823796e+305
            # cardCAj: 501 0.5 * (cardCAj + 1) - 1: 250.0
            # -> setting it to almost 0.

        else:
            varCAj = 1.0 / np.random.gamma(.5, 2.)
            if cardCAj == 0:
                meanCAj = np.random.normal(5.0, varCAj ** 0.5)
            else:
                meanCAj = np.random.normal(self.nrlCA[j], varCAj ** 0.5)

        logger.info('Sampled components - cond: %d', j)
        logger.info('var CI = %f', varCIj)
        logger.info('mean CA = %f, var CA = %f', meanCAj, varCAj)

        return varCIj, meanCAj, varCAj

    def sampleNextInternal(self, variables):
        # TODO : comment

        nrlsSmpl = self.samplerEngine.get_variable('nrl')

        cardCA = nrlsSmpl.cardClass[self.L_CA, :]
        cardCI = nrlsSmpl.cardClass[self.L_CI, :]

        for j in xrange(self.nbConditions):
            vICI = nrlsSmpl.voxIdx[nrlsSmpl.L_CI][j]
            vICA = nrlsSmpl.voxIdx[nrlsSmpl.L_CA][j]
            self.nrlCI[j] = nrlsSmpl.currentValue[j, vICI]
            self.nrlCA[j] = nrlsSmpl.currentValue[j, vICA]

        for j in xrange(self.nbConditions):
            # for j in np.random.permutation(self.nbConditions):
            if self.hyperPriorFlag:
                varCIj, meanCAj, varCAj = self.computeWithProperPriors(j,
                                                                       cardCI[
                                                                           j],
                                                                       cardCA[j])
            else:
                varCIj, meanCAj, varCAj = self.computeWithJeffreyPriors(j,
                                                                        cardCI[
                                                                            j],
                                                                        cardCA[j])

            self.currentValue[self.I_VAR_CI, j] = varCIj
            self.currentValue[self.I_MEAN_CA, j] = meanCAj  # absolute(meanCAj)
            self.currentValue[self.I_VAR_CA, j] = varCAj

            logger.debug('varCI,%d=%f', j, self.currentValue[self.I_VAR_CI, j])
            logger.debug(
                'meanCA,%d=%f', j, self.currentValue[self.I_MEAN_CA, j])
            logger.debug(
                'varCA,%d = %f', j, self.currentValue[self.I_VAR_CA, j])

    def updateObsersables(self):
        GibbsSamplerVariable.updateObsersables(self)
        sHrf = self.samplerEngine.get_variable('hrf')
        sScale = self.samplerEngine.get_variable('scale')

        if sHrf.sampleFlag and np.allclose(sHrf.normalise, 0.) and \
                not sScale.sampleFlag and self.sampleFlag:
            logger.debug('Normalizing Posterior mean of '
                         'Mixture Parameters at each iteration ...')
            # Undo previous calculation:
            self.cumul -= self.currentValue
            self.cumul3 -= (self.currentValue - self.mean) ** 2

            # Use scaled quantities instead:
            cur_m_CA = self.currentValue[self.I_MEAN_CA]
            cur_v_CA = self.currentValue[self.I_VAR_CA]
            cur_v_CI = self.currentValue[self.I_VAR_CI]
            self.cumul[self.I_MEAN_CA] += cur_m_CA * sHrf.norm
            self.cumul[self.I_VAR_CA] += cur_v_CA * sHrf.norm ** 2
            self.cumul[self.I_VAR_CI] += cur_v_CI * sHrf.norm ** 2

            self.mean = self.cumul / self.nbItObservables

            self.cumul3[
                self.I_MEAN_CA] += (cur_m_CA * sHrf.norm - self.mean[self.I_MEAN_CA]) ** 2
            self.cumul3[
                self.I_VAR_CA] += (cur_v_CA * sHrf.norm ** 2 - self.mean[self.I_VAR_CA]) ** 2
            self.cumul3[
                self.I_VAR_CI] += (cur_v_CI * sHrf.norm ** 2 - self.mean[self.I_VAR_CI]) ** 2

            self.error = self.cumul3 / self.nbItObservables

    def get_string_value(self, v):
        v = v.transpose()
        if 0:
            print 'get_string_value for mixt_params ...'
            print v.shape, self.dataInput.cNames
            print '->', v[:, :len(self.dataInput.cNames)].shape
        return get_2Dtable_string(v[:len(self.dataInput.cNames), :],
                                  self.dataInput.cNames,
                                  self.PARAMS_NAMES,)

    def getOutputs(self):
        outputs = {}
        if pyhrf.__usemode__ == pyhrf.DEVEL:
            outputs = GibbsSamplerVariable.getOutputs(self)
            mixtp = np.zeros((2, self.nbConditions, 2))
            mixtp[self.L_CA, :, 0] = self.finalValue[self.I_MEAN_CA, :]
            mixtp[self.L_CA, :, 1] = self.finalValue[self.I_VAR_CA, :]
            mixtp[self.L_CI, :, 0] = 0.
            mixtp[self.L_CI, :, 1] = self.finalValue[self.I_VAR_CI, :]

            an = ['class', 'condition', 'component']
            ad = {'class': ['inactiv', 'activ'], 'condition': self.dataInput.cNames,
                  'component': ['mean', 'var']}
            outputs['pm_' + self.name] = xndarray(mixtp, axes_names=an,
                                                  axes_domains=ad)

            mixtp_mapped = np.tile(mixtp, (self.nbVox, 1, 1, 1))
            outputs['pm_' + self.name + '_mapped'] = xndarray(mixtp_mapped,
                                                              axes_names=[
                                                                  'voxel'] + an,
                                                              axes_domains=ad)

            region_is_active = self.finalValue[self.I_MEAN_CA, :].max() > \
                self.activ_thresh
            region_is_active = region_is_active.astype(np.int16)
            region_is_active = np.tile(region_is_active, self.nbVox)

            an = ['voxel']
            outputs['active_regions_from_mean_activ'] = xndarray(region_is_active,
                                                                 axes_names=an)

        return outputs

    def finalizeSampling(self):
        GibbsSamplerVariable.finalizeSampling(self)
        del self.nrlCA
        del self.nrlCI



# -*- coding: utf-8 -*-

import unittest
import os
import os.path as op
import shutil
import logging

import numpy as np
import numpy.testing as npt

import pyhrf

from pyhrf.xmlio import XmlInitable, to_xml, from_xml
from pyhrf import xmlio


logger = logging.getLogger(__name__)


class A(XmlInitable):

    def __init__(self, p=1, c='a'):
        XmlInitable.__init__(self)
        self.param1 = p
        self.param2 = c

    def __eq__(self, o):
        return isinstance(o, A) and o.param1 == self.param1 and \
            o.param2 == self.param2


class TestXML(unittest.TestCase):

    def setUp(self):
        self.tmp_dir = pyhrf.get_tmp_path()

    def tearDown(self):
        shutil.rmtree(self.tmp_dir)

    def test_simple_bijection(self):
        a = A(c='e')
        a_xml = to_xml(a)
        self.assertEqual(a, from_xml(a_xml))


class BaseTest(unittest.TestCase):

    def test_basic_types(self):

        dictObj = {'clef1': 'val1', 'clef2': 2}
        sXml = xmlio.to_xml(dictObj)
        dictObj2 = xmlio.from_xml(sXml)
        assert dictObj2 == dictObj

        obj = ['ggg', 5, (677, 56), 4.5, 'ff']
        sXml = xmlio.to_xml(obj)
        obj2 = xmlio.from_xml(sXml)
        assert obj2 == obj

    def _test_xml_bijection(self, o):
        xml = to_xml(o)
        logger.debug('xml from %s:', str(o))
        logger.debug(xml)
        o2 = from_xml(xml)
        self.assertEqual(o, o2)

    def test_list_of_str(self):
        self._test_xml_bijection(['a', 'bb', 'ccc', ''])

    def test_bool(self):
        self._test_xml_bijection(False)

    def test_list_of_int(self):
        self._test_xml_bijection([-1, 134, 1])

    def test_list_of_misc(self):
        self._test_xml_bijection([-1, None, 'aa', True, False])

    def test_tuple_of_misc(self):
        self._test_xml_bijection((-1, None, 'aa', True, False))

    def testNumpy(self):

        obj = \
            {'array1': np.random.randn(10),
             'val1': np.int(5),
             'array2D': np.ones((4, 4)),
             }
        sXml = xmlio.to_xml(obj)
        obj2 = xmlio.from_xml(sXml)
        assert obj2.keys() == obj.keys()
        assert np.allclose(obj2["array1"], obj["array1"])

    def test_ordered_dict(self):

        try:
            from collections import OrderedDict
        except ImportError:
            from pyhrf.tools.backports import OrderedDict

        d = OrderedDict([('a', 1), ('b', 2)])
        sXml = xmlio.to_xml(d)
        d2 = xmlio.from_xml(sXml)

        self.assertEqual(d, d2)


class TopClass(xmlio.XmlInitable):
    def __init__(self, p_top='1'):
        xmlio.XmlInitable.__init__(self)


class ChildClass(xmlio.XmlInitable):
    def __init__(self, p_child=2):
        xmlio.XmlInitable.__init__(self)


class D(xmlio.XmlInitable):
    def __init__(self, p=2):
        xmlio.XmlInitable.__init__(self)


class T(xmlio.XmlInitable):

    def __init__(self, param_a=1):
        xmlio.XmlInitable.__init__(self)
        self.pa = param_a

    def from_param_c(self, param_c=np.array([56])):
        return T(param_c)
    from_param_c = classmethod(from_param_c)


def create_t():
    return T(param_a=49)


class XMLableTest(unittest.TestCase):

    def testDynamicParamsSingleClass(self):
        o = TopClass()
        self.assertEqual(len(o._init_parameters), 1)
        self.assertEqual(o._init_parameters['p_top'], '1')

    def testDynamicParamsHierachic(self):

        o = ChildClass()
        self.assertEqual(len(o._init_parameters), 1)
        self.assertEqual(o._init_parameters['p_child'], 2)

    def test_set_init_param(self):
        a = A()
        a.set_init_param('c', 'new_c')
        xml = xmlio.to_xml(a, pretty=True)
        new_a = xmlio.from_xml(xml)
        self.assertEqual(new_a.param2, 'new_c')


class B(xmlio.Initable):
    def __init__(self, obj_t=np.array([5])):
        xmlio.Initable.__init__(self)
        self.obj = obj_t

    @classmethod
    def from_stuff(self, a=2, b=5):
        o = B(np.array([a, b]))
        o.set_init(B.from_stuff, a=a, b=b)
        return o

    def __eq__(self, b):
        return (self.obj == b.obj).all()


class C(xmlio.Initable):
    def __init__(self):
        xmlio.Initable.__init__(self)

    def __eq__(self, o):
        return isinstance(o, C)


class InitableTest(unittest.TestCase):

    def test_init(self):
        b = B()
        npt.assert_array_equal(b.obj, np.array([5]))

    def test_pickle_classmethod(self):
        import pickle
        f = pyhrf.core.FmriData.from_surf_files
        s = pickle.dumps(f)
        fs = pickle.loads(s)
        self.assertEqual(f, fs)

    def test_classmethod_init(self):
        b2 = B.from_stuff()
        npt.assert_array_equal(b2.obj, np.array([2, 5]))

    def test_xml_from_init(self):
        b = B()
        xml = b.to_ui_node('bobj').to_xml(pretty=True)
        import re
        pat = '.*<bobj pickled_init_obj="[^>]*test_xml[^>]*B[^>]*"[^>]*>'\
            '.*<obj_t[^>]*pickled_type="[^>]*ndarray[^>]*"[^>]*>5'\
            '</obj_t>.*</bobj>.*'
        if re.match(pat, xml, re.DOTALL) is None:
            raise Exception('Wrong XML :\n%s' % xml)

    def test_xml_from_classmethod_init(self):
        b = B.from_stuff(a=4, b=3)
        np.testing.assert_equal(b.obj, [4, 3])
        xml = b.to_ui_node('bobj').to_xml(pretty=True)
        pat = '.*<bobj pickled_init_obj="[^>]*from_stuff[^>]*"[^>]*>'\
            '.*<a[^>]*pickled_type="[^>]*int[^>]*"[^>]*>4</a>'\
            '.*<b[^>]*pickled_type="[^>]*int[^>]*"[^>]*>3</b>'\
            '.*</bobj>.*'
        import re
        if re.match(pat, xml, re.DOTALL) is None:
            raise Exception('Wrong XML :\n%s' % xml)

    def test_bijection_from_init(self):
        b = B()
        xml = b.to_ui_node('bobj').to_xml(pretty=True)
        b2 = xmlio.from_xml(xml)
        self.assertEqual(b, b2)

    def test_bijection_from_init_no_arg(self):
        o = C()
        xml = o.to_ui_node('bobj').to_xml(pretty=True)
        o2 = xmlio.from_xml(xml)
        self.assertEqual(o, o2)

    def test_bijection_from_classmethod_init(self):
        b = B.from_stuff(a=4, b=3)
        xml = b.to_ui_node('bobj').to_xml(pretty=True)
        b2 = xmlio.from_xml(xml)
        self.assertEqual(b, b2)

    def test_JDEMCMCAnalyzer_Uinode_bijection(self):
        from pyhrf.ui.jde import JDEMCMCAnalyser
        from pyhrf.jde.models import BOLDGibbsSampler
        a = JDEMCMCAnalyser(sampler=BOLDGibbsSampler(nb_iterations=42))
        a_ui = xmlio.UiNode.from_py_object('analyse', a)
        it_node = a_ui.get_children()[0].get_children()[0].get_children()[0]
        self.assertEqual(it_node.label(), str(42))

        a2 = xmlio.Initable.from_ui_node(a_ui)
        self.assertEqual(a2.sampler.nbIterations, 42)

    def test_JDEMCMCAnalyzerXML(self):
        from pyhrf.ui.jde import JDEMCMCAnalyser
        from pyhrf.jde.models import BOLDGibbsSampler
        a = JDEMCMCAnalyser(sampler=BOLDGibbsSampler(nb_iterations=42))
        axml = xmlio.to_xml(a, pretty=True)
        a2 = xmlio.from_xml(axml)
        self.assertEqual(a2.sampler.nbIterations, 42)

    def test_TreatmentXML(self):
        from pyhrf.ui.jde import JDEMCMCAnalyser
        from pyhrf.jde.models import BOLDGibbsSampler
        from pyhrf.ui.treatment import FMRITreatment
        a = JDEMCMCAnalyser(sampler=BOLDGibbsSampler(nb_iterations=42))
        t = FMRITreatment(analyser=a)
        txml = xmlio.to_xml(t, pretty=True)
        t2 = xmlio.from_xml(txml)
        self.assertEqual(t2.analyser.sampler.nbIterations, 42)



# -*- coding: utf-8 -*-
import unittest

from valid_rndm_field import *
from valid_beta_estim import *
#from valid_jde_bold_multi_subj_mono_sess import *

from valid_jde_asl import *
from valid_jde_asl_physio import *
from valid_jde_vem_asl import *





# -*- coding: utf-8 -*-

import os
import os.path as op
import copy as copyModule
import logging

import numpy as np

import pyhrf

from pyhrf.ui.analyser_ui import FMRIAnalyser
from pyhrf.jde.beta import BetaSampler
from pyhrf.jde.nrl.bigaussian import NRLSampler  # , NRLSamplerWithRelVar
from pyhrf.jde.models import BOLDGibbsSampler
from pyhrf.xmlio import XmlInitable
from pyhrf.tools._io import read_volume


logger = logging.getLogger(__name__)

DEFAULT_CFG_FILE = 'detectestim.xml'

distribName = 'pyhrf'


class JDEAnalyser(FMRIAnalyser):

    def __init__(self, outputPrefix='jde_', pass_error=True):
        FMRIAnalyser.__init__(self, outputPrefix, pass_error=pass_error)

    def get_label(self):
        return 'pyhrf_JDE_analysis'


class JDEMCMCAnalyser(JDEAnalyser):
    """
    Class that wraps a JDE Gibbs Sampler to launch an fMRI analysis
    TODO: remove parameters about dt and osf (should go in HRF Sampler class),
    drift (should go in Drift Sampler class)
    """
    P_SAMPLER = 'sampler'
    P_OSFMAX = 'osfMax'         # over-sampling factor
    P_DTMIN = 'dtMin'
    P_DT = 'dt'
    P_DRIFT_LFD_PARAM = 'driftParam'
    P_DRIFT_LFD_TYPE = 'driftType'
    P_RANDOM_SEED = 'randomSeed'

    if pyhrf.__usemode__ == pyhrf.DEVEL:
        parametersToShow = [P_DT, P_DTMIN, P_DRIFT_LFD_TYPE, P_DRIFT_LFD_PARAM,
                            P_RANDOM_SEED, P_SAMPLER]
    elif pyhrf.__usemode__ == pyhrf.ENDUSER:
        parametersToShow = [P_DTMIN, P_DT, P_DRIFT_LFD_TYPE,
                            P_DRIFT_LFD_PARAM, P_SAMPLER]

    parametersComments = {
        P_DTMIN: 'Minimum time resolution for the oversampled estimated signal',
        P_DT: 'If different from 0 or None:\nactual time resolution for the '
        'oversampled estimated signal (%s is ignored).\n Better when '
        'it\'s a multiple of the time of repetition' % P_DTMIN,
        P_DRIFT_LFD_PARAM: 'Parameter of the drift modelling.\nIf drift is '
        '"polynomial" then this is the order of the '
        'polynom.\nIf drift is "cosine" then this is the '
        'cut-off period in second.',
        P_DRIFT_LFD_TYPE: 'Either "cosine" or "polynomial" or "None"',
        P_SAMPLER: 'Set of parameters for the sampling scheme',
    }

    def __init__(self, sampler=BOLDGibbsSampler(), osfMax=4, dtMin=.4,
                 dt=.6, driftParam=4, driftType='polynomial',
                 outputPrefix='jde_mcmc_', randomSeed=None, pass_error=True,
                 copy_sampler=True):

        XmlInitable.__init__(self)
        JDEAnalyser.__init__(self, outputPrefix, pass_error=pass_error)

        self.sampler = sampler
        self.osfMax = osfMax
        self.dtMin = dtMin
        self.dt = dt
        self.driftLfdParam = driftParam
        self.driftLfdType = driftType
        self.copy_sampler = copy_sampler

    def enable_draft_testing(self):
        self.sampler.set_nb_iterations(3)

    def analyse_roi(self, atomData):
        """
        Launch the JDE Gibbs Sampler on a parcel-specific data set *atomData*
        Args:
            - atomData (pyhrf.core.FmriData): parcel-specific data
        Returns:
            JDE sampler object
        """

        if self.copy_sampler:
            sampler = copyModule.deepcopy(self.sampler)
        else:
            sampler = self.sampler
        sInput = self.packSamplerInput(atomData)
        sampler.linkToData(sInput)

        logger.info('Treating region %d', atomData.get_roi_id())
        sampler.runSampling(atomData)
        logger.info('Cleaning memory ...')
        sampler.dataInput.cleanMem()
        return sampler

    def packSamplerInput(self, roiData):

        try:
            shrf = self.sampler.get_variable('hrf')
        except KeyError:
            shrf = self.sampler.get_variable('brf')

        hrfDuration = shrf.duration
        zc = shrf.zc

        simu = None

        if simu != None and shrf.sampleFlag == 0:
            hrfDuration = (len(simu.hrf.get_hrf(0, 0)) - 1) * simu.hrf.dt
            logger.debug('Found simulation data and hrf is not sampled, '
                         'setting hrfDuration to: %s', str(hrfDuration))

        logger.info('building BOLDSamplerInput ...')

        if simu == None or shrf.sampleFlag:
            dt = self.dt if (
                self.dt != None and self.dt != 0.) else -self.dtMin
        elif simu != None and shrf.sampleFlag == 0:
            dt = simu.hrf.dt

        samplerInput = self.sampler.inputClass(roiData, dt=dt,
                                               typeLFD=self.driftLfdType,
                                               paramLFD=self.driftLfdParam,
                                               hrfZc=zc,
                                               hrfDuration=hrfDuration)
        return samplerInput

    if 0:
        def handle_mask(self, mask_file, bold_files, tr,
                        onsets, durations, output_dir, mesh_file=None):

            if mesh_file is None:  # Volumic
                if self.force_input_parcellation:
                    if not op.exists(mask_file):
                        raise IOError("Input parcellation is forced but "
                                      "mask file %s not found" % mask_file)
                    else:
                        # TODO: check if n-ary
                        return mask_file

                FMRIAnalyser.handle_mask(self, mask_file, bold_files, onsets,
                                         durations, mesh_file)

                mask, mask_obj = read_volume(mask_file)
                roi_ids = np.unique(mask)
                if len(roi_ids) <= 2:
                    glm_output_dir = op.join(output_dir, 'GLM')
                    if not op.exists(glm_output_dir):
                        os.makedirs(glm_output_dir)
                    return glm_parcellation(bold_file, tr)

#######################################
# Some functions used in scripts      #
# -> used to parallelize calculation  #
#######################################

from pyhrf.jde.models import BOLDGibbsSampler
from pyhrf.jde.hrf import HRFSampler, RHSampler  # , HRFSamplerWithRelVar


def runEstimationBetaEstim(params):

    sessData = params['data']
    nbIterations = params['nbIterations']
    histPace = params.get('histPace', -1)
    sigma = params.get('sigma', 1.5)
    prBetaCut = params.get('prBetaCut', 2.5)
    sampleBetaFlag = params.get('sampleBetaFlag', True)
    pfmethod = params.get('PFMethod', 'ps')
    gridlnz = params.get('gridLnZ', None)
    sampleHRF = params.get('sampleHRF', True)
    # pyhrf.verbose.set_verbosity(params.get('verbose', 1))
    # pyhrf.logger.setLevel(params.get('verbose', logging.WARNING))

    betaSampler = BetaSampler({
        BetaSampler.P_VAL_INI: [0.5],
        BetaSampler.P_SAMPLE_FLAG: 1,
        BetaSampler.P_PARTITION_FUNCTION_METH: pfmethod,
        BetaSampler.P_PARTITION_FUNCTION: gridlnz,
    })

    hrfSampler = HRFSampler({
        HRFSampler.P_SAMPLE_FLAG: sampleHRF,
    })

    hrfVarSampler = RHSampler({
        RHSampler.P_SAMPLE_FLAG: 1,
    })

    sampler = BOLDGibbsSampler({
        BOLDGibbsSampler.P_NB_ITERATIONS: nbIterations,
        BOLDGibbsSampler.P_BETA: betaSampler,
        BOLDGibbsSampler.P_HRF: hrfSampler,
        BOLDGibbsSampler.P_RH: hrfVarSampler,
    })

    analyser = JDEAnalyser(sampler=sampler, dt=0.5)

    result = analyser.analyse(sessData)
    output = analyser.outputResults(result, outputResults)

    return output


def runEstimationSupervised(params):

    data = params.get('data')
    nbIterations = params.get('nbIterations', 100)
    histPace = params.get('histPace', -1)
    beta = params['betaSpv']

    sampleHRF = params.get('sampleHRF', True)

    # pyhrf.verbose.set_verbosity(params.get('verbose', 1))
    # pyhrf.logger.setLevel(params.get('verbose', logging.WARNING))

    prmCAm = params.get('prmCAm', 10.)
    prmCAv = params.get('prmCAv', 10.)
    prvCIa = params.get('prvCIa', 3.)
    prvCIb = params.get('prvCIb', 20.)
    prvCAa = params.get('prvCAa', 2.4)
    prvCAb = params.get('prvCAb', 2.8)

    if histPace == None:
        histPace = nbIterations / 10

    betaSampler = BetaSampler({
        BetaSampler.P_SAMPLE_FLAG: 0,
        BetaSampler.P_VAL_INI: np.array([beta]),
    })

    hrfSampler = HRFSampler({
        HRFSampler.P_SAMPLE_FLAG: sampleHRF,
    })

    hrfVarSampler = RHSampler({
        RHSampler.P_SAMPLE_FLAG: 1,
    })

    sampler = BOLDGibbsSampler({
        BOLDGibbsSampler.P_NB_ITERATIONS: nbIterations,
        BOLDGibbsSampler.P_BETA: betaSampler,
        BOLDGibbsSampler.P_RH: hrfVarSampler,
        BOLDGibbsSampler.P_HRF: hrfSampler,
    })

    analyser = JDEAnalyser({
        JDEAnalyser.P_SAMPLER: sampler,
        JDEAnalyser.P_DT: 0.5,
        JDEAnalyser.P_OUTPUT_FORMAT: 'none',
    })

    result = analyser.analyse(data)
    output = analyser.outputResults(result)
    return output


def jde_analyse(data=None, nbIterations=3,
                hrfModel='estimated', hrfNorm=1., hrfTrick=False,
                sampleHrfVar=True,
                hrfVar=1e-5, keepSamples=False, samplesHistPace=1):
    """
    """
    if data is None:
        data = pyhrf.data.get_default_data()
        data.keep_only_rois([2])

    if hrfModel == 'estimated':
        sampleHRF = True
    elif hrfModel == 'canonical':
        sampleHRF = False
    else:
        raise Exception('Unknown hrf model %s' % hrfModel)

    hrfSampler = HRFSampler({
        HRFSampler.P_SAMPLE_FLAG: sampleHRF,
        HRFSampler.P_NORMALISE: hrfNorm,
        HRFSampler.P_TRICK: hrfTrick,
    })

    hrfVarSampler = RHSampler({
        RHSampler.P_SAMPLE_FLAG: sampleHrfVar,
    })

    nrlSampler = NRLSampler({
        NRLSampler.P_KEEP_SAMPLES: keepSamples,
        NRLSampler.P_SAMPLE_HIST_PACE: samplesHistPace,
    })

    sampler = BOLDGibbsSampler({
        BOLDGibbsSampler.P_NB_ITERATIONS: nbIterations,
        BOLDGibbsSampler.P_RH: hrfVarSampler,
        BOLDGibbsSampler.P_HRF: hrfSampler,
        BOLDGibbsSampler.P_NRLS: nrlSampler,
    })

    analyser = JDEAnalyser({
        JDEAnalyser.P_SAMPLER: sampler,
        JDEAnalyser.P_OUTPUT_FILE: None, })
    result = analyser.analyse(data)
    output = analyser.outputResults(result)

    return output



# -*- coding: utf-8 -*-

"""Module designed to the BOLD signal synthesis according to the Linear and
Time Invariant model described in:

  - Makni, S., Ciuciu, P., Idier, J., & Poline, J.-B. (2005). Joint
    detection-estimation of brain activity in functional MRI: a Multichannel
    Deconvolution solution. IEEE Transactions on Signal Processing, 53(9),
    3488–3502. http://doi.org/10.1109/TSP.2005.853303

It also provides a set of plotting functions (based on matplotlib) - see
L{bolsynth.plot}
"""
# import pyhrf.boldsynth.base
# import pyhrf.boldsynth.noise
# import pyhrf.boldsynth.spatialconfig 
# import pyhrf.boldsynth.hrf 
# import pyhrf.boldsynth.nrl 
# import pyhrf.boldsynth.drift 
# import pyhrf.boldsynth.boldmodel 
# import pyhrf.boldsynth.paradigm
#import plot

#import pyhrf.boldsynth.scenarios 



# -*- coding: utf-8 -*-


import sys
import StringIO
import copy as copyModule
import traceback

import pyhrf
from pyhrf import xmlio

from pyhrf.rfir import RFIREstim
from pyhrf.ui.analyser_ui import FMRIAnalyser

DEFAULT_CFG_FILE = './rfir.xml'

class RFIRAnalyser(FMRIAnalyser):

    parametersToShow = ['HrfEstimator']

    def __init__(self, HrfEstimator=RFIREstim(), outputPrefix='hrf_'):
        xmlio.XmlInitable.__init__(self)
        FMRIAnalyser.__init__(self, outputPrefix='rfir_')

        self.hEstimator = HrfEstimator


    def analyse_roi(self, atomData):
        hEstimator = copyModule.deepcopy(self.hEstimator)
        hEstimator.linkToData(atomData)
        hEstimator.run()
        return hEstimator





# -*- coding: utf-8 -*-

import unittest

import numpy as np

from numpy.testing import assert_array_equal, assert_almost_equal

from pyhrf.jde.samplerbase import Trajectory


class TrajectoryTest(unittest.TestCase):

    def test_basic(self):

        v = np.array([1, 2])
        nb_its = 10
        pace = 1
        start = 0
        t = Trajectory(v, ['time'], {'time': np.array([1, 2])},
                       pace, start, nb_its)

        for i in xrange(nb_its):
            v *= 2
            t.record(i)

        assert_array_equal(t.get_last(), np.array([1, 2]) * 2**nb_its)
        self.assertEqual(t.saved_iterations, [-1] + range(start, nb_its))

    def test_burnin(self):

        v = np.array([1, 2])
        nb_its = 10
        pace = 1
        start = 5
        t = Trajectory(v, ['time'], {'time': np.array([1, 2])},
                       pace, start, nb_its, first_saved_iteration=start)

        for i in xrange(nb_its):
            v *= 2
            t.record(i)

        assert_array_equal(t.get_last(), np.array([1, 2]) * 2**nb_its)
        self.assertEqual(t.saved_iterations, range(start, nb_its))


class GibbsTest(unittest.TestCase):

    def test_var_tracking(self):
        pass
